@@ -39,6 +39,7 @@ from _packagedcode.pypi import PythonSetupPyHandler
 from _packagedcode.pypi import SetupCfgHandler
 from _packagedcode.pypi import can_process_dependent_package
 from python_inspector import utils_pypi
+from python_inspector.error import NoVersionsFound
 from python_inspector.setup_py_live_eval import iter_requirements
 from python_inspector.utils_pypi import Environment
 from python_inspector.utils_pypi import PypiSimpleRepository
@@ -192,10 +193,11 @@ def is_valid_version(
     """
     Return True if the parsed_version is valid for the given identifier.
     """
-    if (
-        any(parsed_version not in r.specifier for r in requirements[identifier])
-        or parsed_version in bad_versions
-    ):
+    if parsed_version in bad_versions:
+        return False
+    if any(parsed_version not in r.specifier for r in requirements[identifier]):
+        if all(not r.specifier for r in requirements[identifier]):
+            return True
         return False
     return True
 
@@ -287,10 +289,17 @@ def remove_extras(identifier: str) -> str:
     return name
 
 
+DEFAULT_ENVIRONMENT = utils_pypi.Environment.from_pyver_and_os(
+    python_version="38", operating_system="linux"
+)
+
+
 class PythonInputProvider(AbstractProvider):
-    def __init__(self, environment=None, repos=tuple(), analyze_setup_py_insecurely=False):
+    def __init__(
+        self, environment=DEFAULT_ENVIRONMENT, repos=tuple(), analyze_setup_py_insecurely=False
+    ):
         self.environment = environment
-        self.environment_marker = get_environment_marker_from_environment(environment)
+        self.environment_marker = get_environment_marker_from_environment(self.environment)
         self.repos = repos or []
         self.versions_by_package = {}
         self.dependencies_by_purl = {}
@@ -453,21 +462,25 @@ class PythonInputProvider(AbstractProvider):
         """
         Generate candidates for the given identifier. Overridden.
         """
+        valid_versions = []
         for version in all_versions:
             parsed_version = parse_version(version)
-            if not is_valid_version(
+            if is_valid_version(
                 parsed_version=parsed_version,
                 requirements=requirements,
                 identifier=identifier,
                 bad_versions=bad_versions,
             ):
-                continue
-            yield Candidate(name=name, version=parsed_version, extras=extras)
+                valid_versions.append(parsed_version)
+        if not all(version.is_prerelease for version in valid_versions):
+            valid_versions = [version for version in valid_versions if not version.is_prerelease]
+        for version in valid_versions:
+            yield Candidate(name=name, version=version, extras=extras)
 
     def _iter_matches(
         self,
         identifier: str,
-        requirements: List[Requirement],
+        requirements: Dict,
         incompatibilities: Dict,
     ) -> Generator[Candidate, None, None]:
         """
@@ -476,17 +489,24 @@ class PythonInputProvider(AbstractProvider):
         name = remove_extras(identifier=identifier)
         bad_versions = {c.version for c in incompatibilities[identifier]}
         extras = {e for r in requirements[identifier] for e in r.extras}
+        versions = []
         if not self.repos:
-            all_versions = self.get_versions_for_package(name=name)
-            yield from self.get_candidates(
-                all_versions, requirements, identifier, bad_versions, name, extras
-            )
+            versions.extend(self.get_versions_for_package(name=name))
         else:
             for repo in self.repos:
-                all_versions = self.get_versions_for_package(name=name, repo=repo)
-                yield from self.get_candidates(
-                    all_versions, requirements, identifier, bad_versions, name, extras
-                )
+                versions.extend(self.get_versions_for_package(name=name, repo=repo))
+
+        if not versions:
+            raise NoVersionsFound(f"This package does not exist: {name}")
+
+        yield from self.get_candidates(
+            all_versions=versions,
+            requirements=requirements,
+            identifier=identifier,
+            bad_versions=bad_versions,
+            name=name,
+            extras=extras,
+        )
 
     def find_matches(
         self,
@@ -504,7 +524,11 @@ class PythonInputProvider(AbstractProvider):
 
     def is_satisfied_by(self, requirement: Requirement, candidate: Candidate) -> bool:
         """Whether the given requirement can be satisfied by a candidate. Overridden."""
-        return candidate.version in requirement.specifier
+        if candidate.version in requirement.specifier:
+            return True
+        elif not requirement.specifier:
+            return True
+        return False
 
     def _iter_dependencies(self, candidate: Candidate) -> Generator[Requirement, None, None]:
         """
